/*********************************************************************
 *
 * File:         common.c
 *
 * Description:  NPAC Conferencing System API, common functions for 
 *               session  and application management 
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "common.h"

extern ManagerData managerData;

void nic()
{
}


int readAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = read( fd, buffer, len );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      PRINTF( "readAll error\n" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}


int writeAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = write( fd, buffer, len );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      perror( "writeAll" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}

int sendControlMessage(int sock, MsgHeader *header, char *msg, int len)
{
  char buf[MSG_HEADER_LENGTH];

  buf[0] = header->tag;
  buf[1] = header->session;
  buf[2] = header->sender.handle;
  strcpy(&buf[3], header->sender.user);
  memcpy(&buf[23], &header->sender.host, sizeof(int));
  buf[27] = header->receiver.handle;
  strcpy(&buf[28], header->receiver.user);
  memcpy(&buf[48], &header->receiver.host, sizeof(int));
  memcpy(&buf[52], &len, sizeof(int));

  if(writeAll(sock, buf, MSG_HEADER_LENGTH) < 0)
    fprintf(stderr, "Manager: write error\n");
  if(writeAll(sock, msg, len) < 0)
    fprintf(stderr, "Manager: write error\n");

  return 1;
}

int ncsEngineRequest(short tag, char *data, int len)
{
  MsgHeader header;

  header.tag = tag;
  cuserid(header.sender.user);
  header.sender.host = gethostid();
  header.dataLength = len;

  if(managerData.socket > 0)
    sendControlMessage(managerData.socket, &header, data, len);

  return 1;
}

int getHostID(char *host)
{
  struct hostent *hostinfo;
  
  hostinfo = gethostbyname(host);
  if(hostinfo == NULL)
    {
      return 0;
    }
  
  return *(int *)hostinfo->h_addr;
}

int getHostName(int id, char *host)
{
  struct hostent *hostinfo;
  struct in_addr in;
  char *tmp;
  
  hostinfo = gethostbyaddr(&id, sizeof(int), AF_INET);
  if(hostinfo == NULL)
    {
      in.s_addr = id;
      tmp = inet_ntoa(in);
      if(tmp == NULL)
	return 0;
      strcpy(host, tmp);
      return 1;
    }
  
  strcpy(host, hostinfo->h_name);
  return 1;
}

int readMsgHeader(int sock, MsgHeader *header)
{
  char buf[MSG_HEADER_LENGTH];
  int err;

  PRINTF("Managet: reading header\n");
  err = readAll(sock, buf, MSG_HEADER_LENGTH);
  if(err == 0)
    {
      fprintf(stderr, "Manager: connection broken\n");
      return err;
    }
  if(err < 0)
    {
      PRINTF("Manager: reading error\n");
      return err;
    }

  header->tag = buf[0];
  header->session = buf[1];
  header->sender.handle = buf[2];
  strcpy(header->sender.user, &buf[3]);
  header->sender.host = *(int *)&buf[23];
  header->receiver.handle = buf[27];
  strcpy(header->receiver.user, &buf[28]);
  header->receiver.host = *(int *)&buf[48];
  header->dataLength = *(int *)&buf[52];

  return err;
}

/* The lowest level UDP data packages sending procedure */
int sendPacket(int socket, int hostid, int port, char *msg, int len)
{
  struct sockaddr_in addr;

  bzero(&addr, sizeof(struct sockaddr_in));
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = htonl(hostid);
  addr.sin_port        = htons(port);

  if(sendto(socket, msg, len, 
	 0, &addr, sizeof(struct sockaddr_in)) < 0)
    return 0;  

  return 1;
}


int start_process(char *name)
{
  int pid;

  if((pid = fork()) < 0)
    {
      printf("Manager: fork error\n");
      return -1;
    }
  else if(pid == 0)
    {
      execl(name, 0);
    }
  else
    return pid;
}

